@@ -91,7 +91,7 @@ static int parse_local_endpoint_name(char *buf, size_t buf_len, const char *data
 	return 0;
 }
 
-/* We received an IPA-encapsulated MGCP message from a MSC. msg owned by caller. */
+/* We received an IPA-encapsulated MGCP message from a MSC. Transfers msg ownership. */
 int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 {
 	struct bsc_msc_data *msc;
@@ -107,14 +107,16 @@ int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 	     osmo_ss7_asp_get_name(asp), msg->l2h);
 
 	msc = msc_from_asp(asp);
-	if (!msc)
-		return 0;
+	if (!msc) {
+		rc = 0;
+		goto free_msg_ret;
+	}
 
 	rc = parse_local_endpoint_name(rcv_ep_local_name, sizeof(rcv_ep_local_name), (const char *)msg->l2h);
 	if (rc < 0) {
 		LOGP(DMSC, LOGL_ERROR, "(%s:) Received IPA-encapsulated MGCP: Failed to parse CIC\n",
 		     osmo_ss7_asp_get_name(asp));
-		return rc;
+		goto free_msg_ret;
 	}
 
 	/* Lookup which conn attached to the MSC holds an MGW endpoint with the
@@ -143,7 +145,8 @@ int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 	if (!mgcp_cli) {
 		LOGP(DMSC, LOGL_ERROR, "(%s:) Received IPA-encapsulated MGCP: Failed to find associated MGW\n",
 		     osmo_ss7_asp_get_name(asp));
-		return 0;
+		rc = 0;
+		goto free_msg_ret;
 	}
 
 	rc = osmo_sockaddr_str_from_str(&osa_str, mgcp_client_remote_addr_str(mgcp_cli),
@@ -151,7 +154,7 @@ int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 	if (rc < 0) {
 		LOGP(DMSC, LOGL_ERROR, "(%s:) Received IPA-encapsulated MGCP: Failed to parse MGCP address %s:%u\n",
 		     osmo_ss7_asp_get_name(asp), mgcp_client_remote_addr_str(mgcp_cli), mgcp_client_remote_port(mgcp_cli));
-		return rc;
+		goto free_msg_ret;
 	}
 
 	LOGP(DMSC, LOGL_NOTICE, "%s: Forwarding IPA-encapsulated MGCP to MGW at " OSMO_SOCKADDR_STR_FMT "\n",
@@ -161,7 +164,7 @@ int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 	if (rc < 0) {
 		LOGP(DMSC, LOGL_ERROR, "(%s:) Received IPA-encapsulated MGCP: Failed to parse MGCP address " OSMO_SOCKADDR_STR_FMT "\n",
 		     osmo_ss7_asp_get_name(asp), OSMO_SOCKADDR_STR_FMT_ARGS_NOT_NULL(&osa_str));
-		return rc;
+		goto free_msg_ret;
 	}
 	dest_len = osmo_sockaddr_size(&osa);
 
@@ -169,6 +172,8 @@ int bsc_sccplite_rx_mgcp(struct osmo_ss7_asp *asp, struct msgb *msg)
 	 * to be large enough to deal with whatever small/infrequent MGCP messages */
 	rc = sendto(msc->mgcp_ipa.ofd.fd, msgb_l2(msg), msgb_l2len(msg), 0, &osa.u.sa, dest_len);
 
+free_msg_ret:
+	msgb_free(msg);
 	return rc;
 }
 
@@ -195,6 +200,6 @@ int bsc_sccplite_mgcp_proxy_cb(struct osmo_fd *ofd, unsigned int what)
 	msg->l2h[rc] = '\0';
 	LOGP(DMSC, LOGL_NOTICE, "Received MGCP on UDP proxy socket: %s\n", msg->l2h);
 
-	osmo_ipa_msg_push_header(msg, IPAC_PROTO_MGCP_OLD);
+	ipa_prepend_header(msg, IPAC_PROTO_MGCP_OLD);
 	return bsc_sccplite_msc_send(msc, msg);
 }
